@@ -1,7 +1,7 @@
 /*
  *   fs/cifs/transport.c
  *
- *   Copyright (C) International Business Machines  Corp., 2002,2003
+ *   Copyright (C) International Business Machines  Corp., 2002,2004
  *   Author(s): Steve French (sfrench@us.ibm.com)
  *
  *   This library is free software; you can redistribute it and/or modify
@@ -29,7 +29,7 @@
 #include "cifsglob.h"
 #include "cifsproto.h"
 #include "cifs_debug.h"
-
+  
 extern kmem_cache_t *cifs_mid_cachep;
 extern kmem_cache_t *cifs_oplock_cachep;
 
@@ -151,6 +151,7 @@ smb_send(struct socket *ssocket, struct smb_hdr *smb_buffer,
 	set_fs(get_ds());
 	rc = sock_sendmsg(ssocket, &smb_msg, smb_buf_length + 4);
 	while((rc == -ENOSPC) || (rc == -EAGAIN)) {
+		set_current_state(TASK_INTERRUPTIBLE);
 		schedule_timeout(HZ/2);
 		rc = sock_sendmsg(ssocket, &smb_msg, smb_buf_length + 4);
 	}
@@ -233,15 +234,30 @@ SendReceive(const unsigned int xid, struct cifsSesInfo *ses,
 		timeout = 15 * HZ;
 	/* wait for 15 seconds or until woken up due to response arriving or 
 	   due to last connection to this server being unmounted */
-
-	timeout = wait_event_interruptible_timeout(ses->server->response_q,
-				midQ->
-				midState & MID_RESPONSE_RECEIVED,
-				timeout);
 	if (signal_pending(current)) {
-		cFYI(1, ("CIFS: caught signal"));
+		/* if signal pending do not hold up user for full smb timeout
+		but we still give response a change to complete */
+		if(midQ->midState & MID_REQUEST_SUBMITTED) {
+			set_current_state(TASK_UNINTERRUPTIBLE);
+			timeout = schedule_timeout(2 * HZ);
+		}
+	} else { /* using normal timeout */
+		/* timeout = wait_event_interruptible_timeout(ses->server->response_q,
+			(midQ->midState & MID_RESPONSE_RECEIVED) || 
+			((ses->server->tcpStatus != CifsGood) &&
+			 (ses->server->tcpStatus != CifsNew)),
+			timeout); */ 
+		/* Can not allow user interrupts- wreaks havoc with performance */
+		if(midQ->midState & MID_REQUEST_SUBMITTED) {
+			set_current_state(TASK_UNINTERRUPTIBLE);
+			timeout = schedule_timeout(timeout);
+		}
+	}
+	if (signal_pending(current)) {
+		if (midQ->resp_buf == NULL)
+			rc = -EINTR; /* BB are we supposed to return -ERESTARTSYS ? */
 		DeleteMidQEntry(midQ);
-		return -EINTR;
+		return rc; /* why bother returning an error if it succeeded */
 	} else {  /* BB spinlock protect this against races with demux thread */
 		spin_lock(&GlobalMid_Lock);
 		if (midQ->resp_buf) {
@@ -254,16 +270,18 @@ SendReceive(const unsigned int xid, struct cifsSesInfo *ses,
 				if(ses->server->tcpStatus == CifsExiting)
 					rc = -EHOSTDOWN;
 				else {
-				ses->server->tcpStatus = CifsNeedReconnect;
-				midQ->midState = MID_RETRY_NEEDED;
+					ses->server->tcpStatus = CifsNeedReconnect;
+					midQ->midState = MID_RETRY_NEEDED;
 				}
 			}
 
-			if(midQ->midState == MID_RETRY_NEEDED) {
-				rc = -EAGAIN;
-				cFYI(1,("marking request for retry"));
-			} else {
-				rc = -EIO;
+			if (rc != -EHOSTDOWN) {
+				if(midQ->midState == MID_RETRY_NEEDED) {
+					rc = -EAGAIN;
+					cFYI(1,("marking request for retry"));
+				} else {
+					rc = -EIO;
+				}
 			}
 			spin_unlock(&GlobalMid_Lock);
 			DeleteMidQEntry(midQ);
